function xdot = NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u)
% function xdot = NonlinearMSD(t,x,m,k1,k2,b1,b2,F,u)
% 
% This function implements the ordinary differential equation for the
% nonlinear mass-spring-damper system.  The system consists of M blocks 
% connected by nonlinear springs and dampers.  In addition, each block
% is connected to a wall by a nonlinear spring and damper. The system has 
% an input force u and an exogeneous force F on the M^th block.
%
% Inputs:
%  *t = time, sec
%  *x = 2M-by-1 vector specifying the state of the M block, nonlinear 
%       mass-spring-damper system as:
%          x = [p1; p2; ...; pM; v1; v2;....; vM]
%       where (pi,vi) are the position (m) and velocity (m/s) of block i.
%  *m = Mass of individual blocks, kg
%  *k1 = Linear spring constant, N/m
%  *k2 = Cubic spring constant, N/m^3
%  *b1 = Linear damping constant, N/(m/s)
%  *b2 = Cubic damping constant, N/(m/s)^3
%  *F = Nf-by-2 matrix specifying the force (in N) on the M^th block. 
%       The i^th row of F specifies a time F(i,1) and force F(i,2). The
%       force applied at time t in the simulation is computed by 
%       interpolating this matrix of data.
%  *u = Nu-by-2 matrix specifying the force (in N) on the M^th block.
%       The i^th row of u specifies a time u(i,1) and force u(i,2). 
%       The force applied at time t in the simulation is computed by 
%       interpolating this matrix of data.
%
% Output:
%  *xdot = 2M-by-1 vector specifying the derivative of the system:
%          xdot = [v1; v2;....; vM; a1; a2; ...; aM]
%       where (vi,ai) are the velocity (m/s) and acceleration (m/s^2)
%       of block i.
%

% # of Masses
M = numel(x)/2;

% Extract positions and velocities
p = x(1:M);
v = x(M+1:end);

% Compute the block accelerations
a = zeros(M,1);
for i=1:M
    if i==1 && M==1
        % Special Case: Only a single block

        % Input force on block M at time t
        ut = interp1(u(:,1),u(:,2),t);
        
        % Input force on block M=1 at time t
        Ft = interp1(F(:,1),F(:,2),t);
        
        % Total force on mass
        Ftot = ut + Ft;        
    elseif i==1                        
        % Total force on mass
        Ftot = Fs(p(2)-p(1),k1,k2) + Fd(v(2)-v(1),b1,b2);
    elseif i==M
        % Input force on block M at time t
        ut = interp1(u(:,1),u(:,2),t);
                
        % External force on block M at time t
        Ft = interp1(F(:,1),F(:,2),t);

        % Total force on mass
        Ftot = ut + Ft - Fs(p(M)-p(M-1),k1,k2) - Fd(v(M)-v(M-1),b1,b2);
    else       
        % Total force on mass
        Ftot = - Fs(p(i)-p(i-1),k1,k2) - Fd(v(i)-v(i-1),b1,b2) ...
                + Fs(p(i+1)-p(i),k1,k2) + Fd(v(i+1)-v(i),b1,b2);
    end
    
    % Connect each block by an additional spring/damper to wall
    Ftot = Ftot - Fs(p(i),k1,k2) - Fd(v(i),b1,b2);
    
    % Acceleration
    a(i) = Ftot/m;
end

% State derivative 
xdot = [v; a];



%% Local function to compute spring force
function y = Fs(u,k1,k2)
y = k1*u+k2*u^3;

%% Local function to compute damping force
function y = Fd(u,b1,b2)
y = b1*u+b2*u^3;
